/**
 * JASMINe
 * Copyright (C) 2011 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: IndicatorListCommand.java 8788 2011-09-01 15:00:57Z durieuxp $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.probe.shell;

import java.io.PrintStream;
import java.util.Collection;
import java.util.Iterator;
import java.util.Set;

import org.apache.felix.ipojo.annotations.Component;
import org.apache.felix.ipojo.annotations.Provides;
import org.apache.felix.ipojo.annotations.Requires;
import org.apache.felix.shell.Command;
import org.ow2.jasmine.probe.JasmineIndicator;
import org.ow2.jasmine.probe.JasmineProbeManager;

/**
 * Implements indicator-list command that list the name of all the created indicators.
 * @author danesa
 */
@Component(name="IndicatorListCommand")
@Provides
public class IndicatorListCommand extends JasmineProbeCommand implements Command {

        /**
         * command name
         */
        private String name = "indicator-list";

        /**
         * command description
         */
        private String description = "List all JASMINe Indicators, or show a particular Indicator";


        // ----------------------------------------------------------------------------
        // ipojo management
        // ----------------------------------------------------------------------------

        @Requires
        JasmineProbeManager probeManager = null;

        // ----------------------------------------------------------------------------
        // constructors
        // ----------------------------------------------------------------------------

        /**
         * Default Constructor
         */
        public IndicatorListCommand() {
            super();
            setParams(new IndicatorListCommandParams());
        }

        // ----------------------------------------------------------------------------
        // org.apache.felix.shell.Command implementation
        // ----------------------------------------------------------------------------

        /**
         * Returns the name of the command that is implemented by the interface.
         * The command name should not contain whitespace and should also be unique.
         * @return the name of the command.
         */
        public String getName() {
            return name;
        }

        /**
         * Returns a short description of the command; this description should be as short as possible.
         * This information is used when generating the command help information.
         * @return a short description of the command.
         */
        public String getShortDescription() {
            return description;
        }

        public JasmineProbeManager getProbeManager() {
            return probeManager;
        }

        /**
         * Executes the command using the supplied command line, output print stream, and error print stream.
         * @param line the complete command line, including the command name.
         * @param out  the print stream to use for standard output.
         * @param err  the print stream to use for standard error.
         */
        public void execute(String line, PrintStream out, PrintStream err) {
            boolean validOptions = parseOptions(line, out, err);
            if (!validOptions) {
                err.println(getUsage());
                return;
            }

            IndicatorListCommandParams params = ((IndicatorListCommandParams) getParams());

            // get command arguments
            String indicatorName = params.getName();
            boolean verbose = params.isVerbose();
            boolean types = params.isTypes();

            if (types) {
                listTypes(out);
            } else {
                if (indicatorName == null) {
                    listIndicators(verbose, out);
                } else {
                    showIndicator(indicatorName, out, err);
                }
            }

            params.reset();
        }

        private void listIndicators(boolean verbose, PrintStream out) {
            Collection<JasmineIndicator> indicators = probeManager.getIndicators();
            if (indicators.isEmpty()) {
                out.println("There are no JasmineIndicators");
                return;
            }
            for (JasmineIndicator indicator : indicators) {
                if (verbose) {
                    out.println(indicator.toString());
                    out.println();
                } else {
                    out.println(indicator.getName());
                }
            }
        }

        private void showIndicator(String indicatorName, PrintStream out, PrintStream err) {
            Collection<JasmineIndicator> indicators = probeManager.getIndicators();
            for (JasmineIndicator indicator : indicators) {
                if (indicatorName.equals(indicator.getName())) {
                    out.println(indicator.toString());
                    return;
                }
            }
            err.println("There is no JasmineIndicator named " + indicatorName);
        }

        private void listTypes(PrintStream out) {

            Set<String> types = probeManager.getIndicatorTypes();
            if (types.isEmpty()) {
                out.println("There are no available JasmineIndicator types");
            } else {
                out.println("Available JasmineIndicator types:");
            }
            for (Iterator<String> itTypes = types.iterator(); itTypes.hasNext(); ) {
                out.println(itTypes.next());
            }
        }
}
